@@ -143,9 +143,9 @@ the exception of those in mlab.py provided by matplotlib.
 from __future__ import generators
 
 
-__version__  = '0.86.2'
-__revision__ = '$Revision: 1.104 $'
-__date__     = '$Date: 2006/01/24 12:56:45 $'
+__version__  = '0.87'
+__revision__ = '$Revision: 1.122 $'
+__date__     = '$Date: 2006/02/22 14:33:56 $'
 
 import sys, os, warnings, shutil, md5
 import distutils.sysconfig
@@ -312,46 +312,6 @@ def _get_home():
 
 
 
-def _old_get_home():  #ignore me
-    """Return the closest possible equivalent to a 'home' directory.
-
-    We first try $HOME.  Absent that, on NT it's USERPROFILE if
-    defined, else, $HOMEDRIVE\$HOMEPATH, else C:\
-
-    """
-
-
-    try: return os.environ['HOME']
-    except KeyError: pass
-
-    if os.name == 'nt':
-        # For some strange reason, win9x returns 'nt' for os.name.
-
-        try: p = os.environ['USERPROFILE']
-        except KeyError: pass
-        else:
-            if os.path.exists(p): return p
-
-        try: p =  os.path.join(os.environ['HOMEDRIVE'],os.environ['HOMEPATH'])
-        except KeyError: pass
-        else:
-            if os.path.exists(p): return p
-
-        try:
-            import _winreg as wreg
-            key = wreg.OpenKey(wreg.HKEY_CURRENT_USER,
-                               'Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders')
-            p = wreg.QueryValueEx(key,'Personal')[0]
-            key.Close()
-            if os.path.exists(p): return p
-        except: pass
-        return 'C:\\'
-
-    raise RuntimeError('please define environment variable $HOME')
-
-
-
-
 get_home = verbose.wrap('$HOME=%s', _get_home, always=False)
 
 def _get_configdir():
@@ -374,67 +334,41 @@ get_configdir = verbose.wrap('CONFIGDIR=%s', _get_configdir, always=False)
 
 def _get_data_path():
     'get the path to matplotlib data'
-    
+
     if os.environ.has_key('MATPLOTLIBDATA'):
         path = os.environ['MATPLOTLIBDATA']
         if os.path.isdir(path): return path
     
-    else:
-        path = os.sep.join([os.path.dirname(__file__), 'mpl-data'])
-        if os.path.isdir(path): return path
+    path = os.sep.join([os.path.dirname(__file__), 'mpl-data'])
+    if os.path.isdir(path): return path
+
+    # setuptools' namespace_packages may highjack this init file
+    # so need to try something known to be in matplotlib, not basemap
+    import matplotlib.afm
+    path = os.sep.join([os.path.dirname(matplotlib.afm.__file__), 'mpl-data'])
+    if os.path.isdir(path): return path
+    
+    # py2exe zips pure python, so still need special check
+    if (sys.platform=='win32' or sys.platform=='darwin') and sys.frozen:
+        path = os.path.join(os.path.split(sys.path[0])[0], 'matplotlibdata')
+        if os.path.isdir(path):  return path
+        else:
+            # Try again assuming sys.path[0] is a dir not a exe
+            path = os.path.join(sys.path[0], 'matplotlibdata')
+            if os.path.isdir(path): return path
 
-#    if _have_pkg_resources:
-#        try:
-#            dist = pkg_resources.get_distribution('matplotlib')
-#        except pkg_resources.DistributionNotFound:
-#            # pkg_resources is installed, but setuptools wasn't used into install matplotlib.
-#            used_setuptools_to_install_MPL = False
-#        else:
-#            used_setuptools_to_install_MPL = True
-#
-#        if used_setuptools_to_install_MPL:
-#            req = pkg_resources.Requirement.parse('matplotlib')
-#            path = pkg_resources.resource_filename(req, 'share/matplotlib')
-#            return path
-#
-#    if os.environ.has_key('MATPLOTLIBDATA'):
-#        path = os.environ['MATPLOTLIBDATA']
-#        if os.path.isdir(path): return path
-#
-#    path = os.path.join(distutils.sysconfig.PREFIX, 'share', 'matplotlib')
-#    if os.path.isdir(path): return path
-#
-#    path = '/usr/local/share/matplotlib'
-#    if os.path.isdir(path): return path
-#
-#    path = '/usr/share/matplotlib'
-#    if os.path.isdir(path): return path
-#
-#    path = os.path.join(os.sep.join(__file__.split(os.sep)[:-1]),
-#                        'share','matplotlib')
-#    if os.path.isdir(path): return path
-#
-#    path = os.path.join(os.sep.join(__file__.split(os.sep)[:-5]),
-#                        'share','matplotlib')
-#    if os.path.isdir(path): return path
-#
-#    # CODE ADDED TO SUPPORT PY2EXE - you will need to copy
-#    # C:\Python23\share\matplotlib into your dist dir.  See
-#    # http://starship.python.net/crew/theller/moin.cgi/MatPlotLib
-#    # for more info
-#
-#    if sys.platform=='win32' and sys.frozen:
-#        path = os.path.join(os.path.split(sys.path[0])[0], 'matplotlibdata')
-#        if os.path.isdir(path):  return path
-#        else:
-#            # Try again assuming sys.path[0] is a dir not a exe
-#            path = os.path.join(sys.path[0], 'matplotlibdata')
-#            if os.path.isdir(path): return path
-#
     raise RuntimeError('Could not find the matplotlib data files')
 
 get_data_path = verbose.wrap('matplotlib data path %s', _get_data_path, always=False)
 
+def get_py2exe_datafiles():
+    import glob
+    
+    mplfiles = glob.glob(os.sep.join([matplotlib.get_data_path(), '*']))
+    # Need to explicitly remove cocoa_agg files or py2exe complains
+    mplfiles.remove(os.sep.join([matplotlib.get_data_path(), 'Matplotlib.nib']))
+    
+    return ('matplotlibdata', mplfiles)
 
 def checkdep_dvipng():
     try:
@@ -443,7 +377,7 @@ def checkdep_dvipng():
         v = line.split()[-1]
         float(v)
         return v
-    except IndexError, ValueError:
+    except (IndexError, ValueError):
         return None
 
 def checkdep_ghostscript():
@@ -458,7 +392,7 @@ def checkdep_ghostscript():
         vtest = '.'.join(v.split('.')[:2]) # deal with version numbers like '7.07.1'
         float(vtest)
         return vtest
-    except IndexError, ValueError:
+    except (IndexError, ValueError):
         return None
 
 def checkdep_ps2eps():
@@ -468,7 +402,7 @@ def checkdep_ps2eps():
         v = line.split()[-1]
         float(v)
         return v
-    except IndexError, ValueError:
+    except (IndexError, ValueError):
         return None
 
 def checkdep_tex():
@@ -482,7 +416,7 @@ def checkdep_tex():
                 float(v)
                 break # found version number
         return v
-    except IndexError, ValueError:
+    except (IndexError, ValueError):
         return None
 
 def checkdep_xpdf():
@@ -492,7 +426,7 @@ def checkdep_xpdf():
         v = line.split()[-1]
         float(v)
         return v
-    except IndexError, ValueError:
+    except (IndexError, ValueError):
         return None
 
 def compare_versions(a, b):
@@ -656,52 +590,48 @@ validate_ps_papersize = ValidateInStrings([
 
 def validate_ps_distiller(s):
     s = s.lower()
-    
+
     if s == 'none':
         return None
     elif s == 'false':
         return False
-    
-    flag = True
-    
-    if (s == 'ghostscript') or (s == 'xpdf'):
+    elif s in ('ghostscript', 'xpdf'):
+        flag = True
         gs_req = '7.07'
         gs_sugg = '8.16'
         gs_v = checkdep_ghostscript()
         if compare_versions(gs_v, gs_sugg): pass
         elif compare_versions(gs_v, gs_req):
-            warnings.warn( 'ghostscript-%s.%s found. ghostscript-%s.%s or later is \
-recommended to use the ps.usedistiller option.' % (gs_v, gs_sugg), 'helpful')
+            verbose.report( 'ghostscript-%s found. ghostscript-%s or later \
+is recommended to use the ps.usedistiller option.' % (gs_v, gs_sugg))
         else:
             flag = False
             warnings.warn('matplotlibrc ps.usedistiller option can not be used \
 unless ghostscript-%s or later is installed on your system'% gs_req)
-    
-    if s == 'xpdf':
-        xpdf_req = '3.0'
-        ps2eps_req = '1.58'
-        xpdf_v = checkdep_xpdf()
-        if compare_versions(xpdf_v, xpdf_req): pass
-        else:
-            flag = False
-            warnings.warn('matplotlibrc ps.usedistiller can not be set to xpdf \
-unless xpdf-%s or later is installed on your system' % xpdf_req)
-        
-        ps2eps_v = checkdep_ps2eps()
-        if compare_versions(ps2eps_v, ps2eps_req): pass
-        else:
-            flag = False
-            warnings.warn('matplotlibrc ps.usedistiller can not be set to xpdf \
+
+        if s == 'xpdf':
+            xpdf_req = '3.0'
+            ps2eps_req = '1.58'
+            xpdf_v = checkdep_xpdf()
+            if compare_versions(xpdf_v, xpdf_req): pass
+            else:
+                flag = False
+                warnings.warn('matplotlibrc ps.usedistiller can not be set to \
+xpdf unless xpdf-%s or later is installed on your system' % xpdf_req)
+
+            ps2eps_v = checkdep_ps2eps()
+            if compare_versions(ps2eps_v, ps2eps_req): pass
+            else:
+                flag = False
+                warnings.warn('matplotlibrc ps.usedistiller can not be set to xpdf \
 unless ps2eps-%s or later is installed on your system' % ps2eps_req)
-        
-        if flag: return s.lower()
+
+        if flag: return s
         else: return None
-    else: 
+    else:
         raise ValueError('matplotlibrc ps.usedistiller must either be none, \
 ghostscript or xpdf')
 
-validate_tex_engine = ValidateInStrings(['tex', 'latex'], ignorecase=True)
-
 def validate_usetex(s):
     bl = validate_bool(s)
     if bl:
@@ -710,31 +640,31 @@ def validate_usetex(s):
         gs_sugg = '8.16'
         dvipng_req = '1.5'
         flag = True
-        
+
         tex_v = checkdep_tex()
         if compare_versions(tex_v, tex_req): pass
         else:
             flag = False
             warnings.warn('matplotlibrc text.usetex option can not be used \
 unless TeX-%s or later is installed on your system' % tex_req)
-        
+
         dvipng_v = checkdep_dvipng()
         if compare_versions(dvipng_v, dvipng_req): pass
         else:
             flag = False
             warnings.warn( 'matplotlibrc text.usetex can not be used with *Agg \
 backend unless dvipng-1.5 or later is installed on your system')
-        
+
         gs_v = checkdep_ghostscript()
         if compare_versions(gs_v, gs_sugg): pass
         elif compare_versions(gs_v, gs_req):
-            warnings.warn( 'ghostscript-%s found. ghostscript-%s or later is \
+            verbose.report( 'ghostscript-%s found. ghostscript-%s or later is \
 recommended for use with the text.usetex option.' % (gs_v, gs_sugg))
         else:
             flag = False
             warnings.warn('matplotlibrc text.usetex can not be used \
 unless ghostscript-%s or later is installed on your system'% gs_req)
-        
+
         if flag: return True
         else: return False
     else:
@@ -813,18 +743,17 @@ defaultParams = {
     'font.variant'      : ['normal', str],           #
     'font.stretch'      : ['normal', str],           #
     'font.weight'       : ['normal', str],           #
-    'font.size'         : ['medium', validate_fontsize], #
+    'font.size'         : [12.0, validate_float], #
     'font.serif'        : ['serif', validate_comma_sep_str],
     'font.sans-serif'   : ['sans-serif', validate_comma_sep_str],
     'font.cursive'      : ['cursive', validate_comma_sep_str],
     'font.fantasy'      : ['fantasy', validate_comma_sep_str],
     'font.monospace'    : ['monospace', validate_comma_sep_str],
-    'font.latex.package': ['type1cm', str],          # LaTeX font package: txfonts, pslatex, ...
 
     # text props
     'text.color'        : ['k', validate_color],     # black
     'text.usetex'       : [False, validate_usetex],
-    'text.tex.engine'   : ['tex', validate_tex_engine], # TeX or LaTeX
+    'text.dvipnghack'    : [False, validate_bool],
     'text.fontstyle'    : ['normal', str],
     'text.fontangle'    : ['normal', str],
     'text.fontvariant'  : ['normal', str],
@@ -917,33 +846,6 @@ defaultParams = {
     }
 
 
-def _old_matplotlib_fname():
-    """
-    Return the path to the rc file using the old search method
-    """
-
-    fname = os.path.join( os.getcwd(), '.matplotlibrc')
-    if os.path.exists(fname): return fname
-
-    if os.environ.has_key('MATPLOTLIBRC'):
-        path =  os.environ['MATPLOTLIBRC']
-        if os.path.exists(path):
-            fname = os.path.join(path, '.matplotlibrc')
-            if os.path.exists(fname):
-                return fname
-
-    home = get_home()
-    if home is not None:
-        fname = os.path.join(home, '.matplotlibrc')
-        if os.path.exists(fname):
-            return fname
-
-    path =  get_data_path() # guaranteed to exist or raise
-    fname = os.path.join(path, '.matplotlibrc')
-    if not os.path.exists(fname):
-        warnings.warn('Could not find .matplotlibrc; using defaults')
-    return fname
-
 def matplotlib_fname():
     """
     Return the path to the rc file
@@ -999,19 +901,44 @@ WARNING: Old rc filename "%s" found and renamed to
     return fname
 
 
+def validate_key(key, val, line, cnt, fname, fail_on_error):
+    if key in _deprecated_map.keys():
+        alt = _deprecated_map[key]
+        warnings.warn('%s is deprecated in matplotlibrc - use %s instead.' % (key, alt))
+        key = alt
+
+    if not defaultParams.has_key(key):
+        warnings.warn('Bad key "%s" on line %d in %s' % (key, cnt, fname))
+        return None
+
+    default, converter =  defaultParams[key]
+
+    ind = val.find('#')
+    if ind>=0: val = val[:ind]   # ignore trailing comments
+    val = val.strip()
+    if fail_on_error:
+        return converter(val)   # try to convert to proper type or raise
+    else:
+        try: cval = converter(val)   # try to convert to proper type or raise
+        except Exception, msg:
+            warnings.warn('Bad val "%s" on line #%d\n\t"%s"\n\tin file "%s"\n\t%s' % (
+                val, cnt, line, fname, msg))
+            return None
+        else:
+            return cval
+
+_deprecated_map = {
+    'text.fontstyle':   'font.style',
+    'text.fontangle':   'font.style',
+    'text.fontvariant': 'font.variant',
+    'text.fontweight':  'font.weight',
+    'text.fontsize':    'font.size',
+    'tick.size' :       'tick.major.size',
+    }
 
 def rc_params(fail_on_error=False):
     'Return the default params updated from the values in the rc file'
 
-    deprecated_map = {
-        'text.fontstyle':   'font.style',
-        'text.fontangle':   'font.style',
-        'text.fontvariant': 'font.variant',
-        'text.fontweight':  'font.weight',
-        'text.fontsize':    'font.size',
-        'tick.size' :       'tick.major.size',
-        }
-
     fname = matplotlib_fname()
     if not os.path.exists(fname):
         message = 'could not find rc file; returning defaults'
@@ -1019,49 +946,35 @@ def rc_params(fail_on_error=False):
         warnings.warn(message)
         return ret
 
+    lines = [line.strip() for line in file(fname)]
     cnt = 0
-    for line in file(fname):
-        cnt +=1
-        line = line.strip()
+    rc_temp = {}
+    for line in lines:
+        cnt += 1
         if not len(line): continue
         if line.startswith('#'): continue
         tup = line.split(':',1)
         if len(tup) !=2:
             warnings.warn('Illegal line #%d\n\t%s\n\tin file "%s"' % (cnt, line, fname))
             continue
-
         key, val = tup
         key = key.strip()
-        if key in deprecated_map.keys():
-            alt = deprecated_map[key]
-            warnings.warn('%s is deprecated in matplotlibrc - use %s instead.' % (key, alt))
-            key = alt
+        rc_temp[key] = (val, line, cnt)
 
-        if not defaultParams.has_key(key):
-            warnings.warn('Bad key "%s" on line %d in %s' % (key, cnt, fname))
-            continue
-
-
-        default, converter =  defaultParams[key]
-
-
-        ind = val.find('#')
-        if ind>=0: val = val[:ind]   # ignore trailing comments
-        val = val.strip()
-        if fail_on_error:
-            cval = converter(val)   # try to convert to proper type or raise
-            defaultParams[key][0] = cval
+    for key in ('verbose.level', 'verbose.fileo'):
+        try: val, line, cnt = rc_temp.pop(key)
+        except KeyError: continue
         else:
-            try: cval = converter(val)   # try to convert to proper type or raise
-            except Exception, msg:
-                warnings.warn('Bad val "%s" on line #%d\n\t"%s"\n\tin file "%s"\n\t%s' % (
-                    val, cnt, line, fname, msg))
-                continue
-            else:
-                # Alles Klar, update dict
-                defaultParams[key][0] = cval
+            cval = validate_key(key, val, line, cnt, fname, fail_on_error)
+            if cval: defaultParams[key][0] = cval
+    while len(rc_temp) > 0:
+        key, (val, line, cnt) = rc_temp.popitem()
+        
+        cval = validate_key(key, val, line, cnt, fname, fail_on_error)
+        if cval: defaultParams[key][0] = cval
+        else: continue
 
-    # strip the conveter funcs and return
+    # strip the converter funcs and return
     ret =  dict([ (key, tup[0]) for key, tup in defaultParams.items()])
     verbose.report('loaded rc file %s'%fname)
 
@@ -1252,7 +1165,7 @@ class ExampleManager:
                 if m is not None:
                     examples.append('%s/%s'%(url, m.group(1)))
         return examples
-            
+
     def get_info(self, s):
         """
         return an ExampleInfo instance from s, the string content of
