@@ -72,7 +72,7 @@ class RendererTemplate(RendererBase):
     def draw_arc(self, gcEdge, rgbFace, x, y, width, height, angle1, angle2):
         pass
     
-    def draw_image(self, x, y, im, origin, bbox):
+    def draw_image(self, x, y, im, bbox):
         pass
     
     def draw_line(self, gc, x1, y1, x2, y2):
@@ -170,6 +170,11 @@ def new_figure_manager(num, *args, **kwargs):
     """
     Create a new figure manager instance
     """
+    # if a main-level app must be created, this is the usual place to
+    # do it -- see backend_wx, backend_wxagg and backend_tkagg for
+    # examples.  Not all GUIs require explicit instantiation of a
+    # main-level app (egg backend_gtk, backend_gtkagg) for pylab
+
     thisFig = Figure(*args, **kwargs)
     canvas = FigureCanvasTemplate(thisFig)
     manager = FigureManagerTemplate(canvas, num)
