@@ -334,7 +334,7 @@ Py::Object
 Bbox::ignore(const Py::Tuple &args) {
   _VERBOSE("Bbox::ignore");
   args.verify_length(1);
-  _ignore = Py::Int(args[1]);
+  _ignore = Py::Int(args[0]);
   return Py::Object();
 }
 
@@ -385,6 +385,33 @@ Bbox::overlapsy(const Py::Tuple &args) {
 }
 
 
+/*
+As for how the datalim handling works, the syntax is
+
+  self.dataLim.update(xys, ignore)
+
+Note this is different than the ax.update_datalim method, which calls
+it.  datalim is a bbox which has an ignore state variable (boolean).
+
+The ignore argument to update datalim can take on three values
+
+  0: do not ignore the current limits and update them with the xys
+  1: ignore the current datalim limits and override with xys
+ -1: use the datalim ignore state to determine the ignore settings
+
+This seems a bit complex but arose from experience.  Basically a lot
+of different objects want to add their data to the datalim.  In most
+use cases, you want the first object to add data to ignore the current
+limits (which are just default values) and subsequent objects to add
+to the datalim taking into account the previous limits.  The default
+behavior of datalim is to set ignore to 1, and after the first call
+with -1 set ignore to 0.  Thus everyone can call with -1 and have the
+desired default behavior .  I hope you are all confused now.
+
+One can manually set the ignore state var with
+
+  datalim.ignore(1)
+*/
 Py::Object
 Bbox::update(const Py::Tuple &args) {
   _VERBOSE("Bbox::update");
@@ -481,7 +508,12 @@ Bbox::update_numerix(const Py::Tuple &args) {
   double maxy = _ur->yval();
 
   double thisx, thisy;
+  //don't use current bounds on first update
   int ignore = Py::Int(args[2]);
+  if (ignore==-1) {
+    ignore = _ignore;
+    _ignore = 0; // don't ignore future updates
+  }
   if (ignore) {
     int xok=0;
     int yok=0;
@@ -505,7 +537,7 @@ Bbox::update_numerix(const Py::Tuple &args) {
 	  yok=1;
 	}
       }
-      
+
       if (xok && yok) break;
     }
   }
@@ -1225,12 +1257,8 @@ SeparableTransformation::operator()(const double& x, const double& y) {
 
   // calling function must first call eval_scalars
 
-  double fx = _funcx->operator()(x);
-  double fy = _funcy->operator()(y);
-
-
-  xy.first  = _sx * fx  +  _tx ;
-  xy.second = _sy * fy  +  _ty;
+  xy.first  = _sx * _funcx->operator()(x) + _tx ;
+  xy.second = _sy * _funcy->operator()(y) + _ty ;
 
   if (_usingOffset) {
     xy.first  += _xot;
@@ -1241,6 +1269,31 @@ SeparableTransformation::operator()(const double& x, const double& y) {
   return xy;
 }
 
+void
+SeparableTransformation::arrayOperator(const int length, const double x[], const double y[], double newx[], double newy[]) {
+  _VERBOSE("SeparableTransformation::arrayOperator");
+
+  _funcx->arrayOperator(length, x, newx);
+  _funcy->arrayOperator(length, y, newy);
+
+  // calling function must first call eval_scalars
+  if (_usingOffset) {
+	for(int i=0; i < length; i++)
+	{
+		newx[i] = (_sx * newx[i]) + _tx + _xot ;
+		newy[i] = (_sy * newy[i]) + _ty + _yot ;
+	}
+  }
+  else {
+	for(int i=0; i < length; i++)
+	{
+		newx[i] = (_sx * newx[i]) + _tx ;
+		newy[i] = (_sy * newy[i]) + _ty ;
+	}
+  }
+
+}
+
 
 std::pair<double, double> &
 SeparableTransformation::inverse_api(const double &x, const double &y) {
@@ -1398,6 +1451,28 @@ NonseparableTransformation::operator()(const double& x, const double& y) {
   return xy;
 }
 
+void
+NonseparableTransformation::arrayOperator(const int length, const double x[], const double y[], double newx[], double newy[]) {
+  _VERBOSE("NonseparableTransformation::operator");
+
+  _funcxy->arrayOperator(length, x, y, newx, newy);
+  if (_usingOffset) {
+	for(int i=0; i < length; i++)
+	{
+		newx[i] = (_sx * newx[i]) + _tx + _xot ;
+		newy[i] = (_sy * newy[i]) + _ty + _yot ;
+	}
+  }
+  else {
+	for(int i=0; i < length; i++)
+	{
+		xy = _funcxy->operator()(x[i], y[i]);
+		newx[i] = (_sx * newx[i]) + _tx ;
+		newy[i] = (_sy * newy[i]) + _ty ;
+	}
+  }
+  return;
+}
 
 std::pair<double, double> &
 NonseparableTransformation::inverse_api(const double &x, const double &y) {
